<?php

namespace Drupal\Tests\dynamic_linsk\Functional;

use Drupal\Core\Url;
use Drupal\Tests\BrowserTestBase;

/**
 * @coversDefaultClass \Drupal\dynamic_links\Controller\DynamicLinksController
 */
class DynamicLinksTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'dynamic_links_test',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * Tests the redirects.
   *
   * @covers ::redirect
   */
  public function testRedirects() {
    $links = [
      [
        'id' => 'front',
        'variants' => [
          [
            'permissions' => ['dynamic_links_test route_1'],
            'expected' => '/dynamic-links-test/route-1',
          ],
          [
            'permissions' => ['dynamic_links_test route_2'],
            'expected' => '/dynamic-links-test/route-2',
          ],
          [
            'permissions' => ['dynamic_links_test route_3'],
            'expected' => '/dynamic-links-test/route-3',
          ],
          [
            'permissions' => [],
            'expected' => '/dynamic-front',
            'code' => 403,
          ],
        ],
      ],
      [
        'id' => 'front_as_routes',
        'variants' => [
          [
            'permissions' => ['dynamic_links_test route_1'],
            'expected' => '/dynamic-links-test/route-1',
          ],
          [
            'permissions' => ['dynamic_links_test route_2'],
            'expected' => '/dynamic-links-test/route-2',
          ],
          [
            'permissions' => ['dynamic_links_test route_3'],
            'expected' => '/dynamic-links-test/route-3',
          ],
          [
            'permissions' => [],
            'expected' => '/dynamic-front/as-routes',
            'code' => 403,
          ],
        ],
      ],
    ];

    foreach ($links as $link) {
      $dynamic_link = Url::fromRoute('dynamic_link.' . $link['id']);
      foreach ($link['variants'] as $data) {
        $user = $this->drupalCreateUser($data['permissions']);
        $this->drupalLogin($user);
        $expected = Url::fromUserInput($data['expected'])->setAbsolute()->toString();
        $code = $data['code'] ?? 200;

        $this->drupalGet($dynamic_link);
        $session = $this->getSession();
        $this->assertEquals($session->getStatusCode(), $code);
        $this->assertEquals($expected, $session->getCurrentUrl());
        $this->drupalLogout();
      }
    }
  }

}
